/*
 *	BIRD Internet Routing Daemon -- The Internet Protocol
 *
 *	(c) 2011 Alexander V. Chernikov <melifaro@ipfw.ru>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 */

#ifndef _BIRD_ADDRS_H_
#define _BIRD_ADDRS_H_

typedef ip_addr ip4_addr;


#ifdef MPLS_VPN
/* Note sizeof(vpn4_addr) is 16 on 64-bit platforms */
typedef struct vpn4_addr {
	u64		rd;
	ip4_addr	addr;
} vpn4_addr;

/* No padding */
typedef struct vpn6_addr {
	u64		rd;
	ip6_addr	addr;
} vpn6_addr;

#endif

/*
 * IANA-define address families:
 * http://www.iana.org/assignments/address-family-numbers/address-family-numbers.xml
 */
#define	IANA_AFI_IP	1
#define	IANA_AFI_IP6	2

/* Buffer must be sufficient to hold RD(15+6+1) + IPv6 address(39) + prefix(4) = 22+39+4=67 */
#define MAX_ADDRESS_P_LENGTH	70

struct fib_node;

#ifdef MPLS_VPN
int addr_print_rd(char *buf, int buflen, u64 rd);
void get_vpn4(void *addrdata, vpn4_addr *newv4);
void get_vpn6(void *addrdata, vpn6_addr *newv6);
void put_vpn4(void *addrdata, vpn4_addr *addr);
void put_vpn6(void *addrdata, vpn6_addr *addr);
#endif
void get_addr(void *addrdata, int rt_family, void *datanew);
void put_addr(void *datanew, int rt_family, void *addrdata);

void addr_print(char *buf, int buflen, int rtype, void *addr);
void fn_print(char *buf, int buflen, struct fib_node *n);

#ifdef MPLS_SUPPORT
/*
 * MPLS labels
 */

#define MPLS_LABEL_IPV4NULL	0
#define MPLS_LABEL_IPV6NULL     2	/* IPv6 Explicit NULL Label */
#define MPLS_LABEL_IMPLNULL     3	/* Implicit NULL Label */
#define MPLS_LABEL_RESERVED_MAX	15

#define MPLS_STACK_DEPTH	8
#define MPLS_STACK_BOS		(1 << 24)

typedef struct bird_mpls_entry {
	u32	local_label;
	u16	action;
	u16	depth;
	u32	stack[MPLS_STACK_DEPTH];
} bird_mpls_entry;

int addr_print_me(char *buf, int bufsize, struct bird_mpls_entry *me, int long_format);

#endif /* MPLS_SUPPORT */

#endif

