/*
 *	BIRD -- LDP Protocol Configuration
 *
 *	(c) 2011 Alexander V. Chernikov <melifaro@ipfw.ru>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 */

CF_HDR

#include "proto/ldp/ldp.h"

CF_DEFINES

#define LDP_CFG ((struct ldp_config *) this_proto)
#define LDP_PATT ((struct ldp_iface_patt *) this_ipatt)
int debug_val, debug_class;

static void
ldp_proto_finish(void)
{
	ldp_check(LDP_CFG);
}

static void
ldp_iface_finish(void)
{
  //struct ldp_iface_patt *ip = LDP_PATT;
}

static void
ldp_set_debug_all(struct ldp_config *lc, int val)
{
	int i;

	for (i = 0; i < LD_MAX; i++)
		lc->debug_mask[i] = val;
}

CF_DECLS

CF_KEYWORDS(LDP, LABEL, SPACE, GLOBAL, PER_INTERFACE, MODE, INDEPENDENT, ORDERED, LOOP, DETECTION)
CF_KEYWORDS(INTERFACE, ECHO, OFF, DISCOVERY, TARGETED, SESSION, HOLD, SEND)
CF_KEYWORDS(NONE, SEVERITY, ERROR, WARNING, NOTICE, VERBOSE, DEBUG, DEBUG2)
CF_KEYWORDS(ALL, GENERAL, HELLO, SESSION, DB, KERNEL)
CF_KEYWORDS(NEIGHBOUR, BINDINGS, FORWARDINGTABLE, INTERFACES, MPLS, ROUTE)
CF_KEYWORDS(LDEBUG)

CF_GRAMMAR

CF_ADDTO(proto, ldp_proto '}' { ldp_proto_finish(); } )

ldp_proto_start: proto_start LDP {
    this_proto = proto_config_new(&proto_ldp, sizeof(struct ldp_config), $1);
    init_list(&LDP_CFG->if_patt_list);

    /* Set some cisco-like defaults */
    LDP_CFG->dhold_interval = 15;
    LDP_CFG->dsend_interval = 5;
    LDP_CFG->thold_interval = 90;
    LDP_CFG->tsend_interval = 10;
    LDP_CFG->skeep_interval = 45;

    /* Debugging defaults */
    ldp_set_debug_all(LDP_CFG, LDP_ERROR * 2 - 1);
  }
 ;

ldp_proto:
   ldp_proto_start proto_name '{'
 | ldp_proto ldp_proto_item ';'
 ;

ldp_proto_item:
   proto_item
 | LABEL SPACE GLOBAL { LDP_CFG->lspace_type = LDP_LABEL_SPACE_GLOBAL; }
 | LABEL SPACE PER_INTERFACE { LDP_CFG->lspace_type = LDP_LABEL_SPACE_INTERFACE; }
 | LOOP DETECTION bool { LDP_CFG->loop_detection = $3; }
 | DISCOVERY HOLD expr {
	 if (($3 < 0) || ($3 > 7200))
		 cf_error("Incorrect discovery hold timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_CFG->dhold_interval = $3;
 }
 | TARGETED HOLD expr {
	 if (($3 < 0) || ($3 > 7200))
		 cf_error("Incorrect targeted hold timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_CFG->thold_interval = $3;
 }
 | DISCOVERY SEND expr {
	 if (($3 < 0) || ($3 > 7200))
		 cf_error("Incorrect discovery hold timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_CFG->dsend_interval = $3;
 }
 | TARGETED SEND expr {
	 if (($3 < 0) || ($3 > 7200))
		 cf_error("Incorrect targeted hold timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_CFG->tsend_interval = $3;
 }
 | SESSION KEEPALIVE expr {
	 if (($3 < 0) || ($3 > 7200))
		 cf_error("Incorrect session keepalive timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_CFG->skeep_interval = $3;
 }
 | MPLS TABLE rtable { LDP_CFG->mpls_table = $3; }
 | ROUTE TABLE rtable { LDP_CFG->route_table = $3; }
 | LDEBUG ldp_debug_mask
 | INTERFACE ldp_iface
 ;

ldp_debug_mask: 
  ldp_debug_mask_item
  | ldp_debug_mask_item ',' ldp_debug_mask
  ;

ldp_debug_mask_item:
  ldp_debug_class ':' ldp_debug_severity {
	  if (debug_class != LD_MAX)
	  	LDP_CFG->debug_mask[debug_class] = debug_val;
	  else
		ldp_set_debug_all(LDP_CFG, debug_val);
  }
  ;

ldp_debug_class:
   GENERAL { debug_class = LD_GENERAL; }
 | HELLO { debug_class = LD_HELLO; }
 | SESSION { debug_class = LD_SESSION; }
 | DB { debug_class = LD_DB; }
 | KERNEL { debug_class = LD_KERN; }
 | ALL { debug_class = LD_MAX; }
 ;

ldp_debug_severity:
   NONE { debug_val = 0; }
 | ERROR { debug_val = LDP_ERROR * 2 - 1; }
 | VERBOSE { debug_val = LDP_VRB * 2 - 1; }
 | WARNING { debug_val = LDP_WRN * 2 - 1; }
 | DEBUG { debug_val = LDP_DBG * 2 - 1; }
 | DEBUG2 { debug_val = LDP_DB2 * 2 - 1; }
 ;

ldp_iface_start:
 {
  this_ipatt = cfg_allocz(sizeof(struct ldp_iface_patt));
  init_list(&this_ipatt->ipn_list);
  add_tail(&LDP_CFG->if_patt_list, NODE this_ipatt);
 }
;

ldp_iface_opts:
  /* empty */
 | ldp_iface_opts ldp_iface_item ';'
 ;

ldp_iface_opt_list:
   /* empty */
 | '{' ldp_iface_opts '}'
 ;

ldp_iface:
  ldp_iface_start iface_patt_list ldp_iface_opt_list { ldp_iface_finish(); }
 ;

ldp_iface_item:
  /* empty */
 | ECHO OFF
 | HOLD expr {
	 if (($2 < 0) || ($2 > 7200))
		 cf_error("Incorrect hold timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_PATT->hold_timer = $2;
	 LDP_PATT->flags |= LDP_FLAG_HOLDTIMER;
 }
 | KEEPALIVE expr {
	 if (($2 < 0) || ($2 > 7200))
		 cf_error("Incorrect keepalive timer value: %d <= MIN < MAX <= %d required", 0, 7200);
	 LDP_PATT->keepalive_timer = $2;
	 LDP_PATT->flags |= LDP_FLAG_KEEPTIMER;
 }
 ;

CF_CLI(SHOW LDP NEIGHBOUR, optsym, [<name>], [[Show details of ldp protocol]])
{ ldp_show_neighbour(proto_get_named($4, &proto_ldp)); } ;
CF_CLI(SHOW LDP INTERFACES, optsym, [<name>], [[Show routing database]])
{ ldp_show_interfaces(proto_get_named($4, &proto_ldp)); } ;
CF_CLI(SHOW LDP BINDINGS, optsym, [<name>], [[Show LDP entries]])
{ ldp_show_bindings(proto_get_named($4, &proto_ldp)); } ;

CF_CODE

CF_END
