/*
 *	BIRD -- Label distribution protocol
 *
 *	(c) 2011 Alexander V. Chernikov <melifaro@ipfw.ru>
 *
 *	Can be freely distributed and used under the terms of the 
 *	GNU GPL or Simplified BSD License.
 */
#ifndef _BIRD_LDP_DATABASE_H_
#define _BIRD_LDP_DATABASE_H_

#define LDP_LIB_FLAG_TLV	0x02 /* One or more TLV is contained inside 'tlvs' list */

typedef struct _ldp_lib_database {
	struct fib		f;		/* fib to store label data */
	int			size;
	list			free_items;	/* List of items currently free */
	int			search_idx;
	int			current_label;	/* Current (used) label */
} ldp_lib_database;

/* RIB */
typedef struct _ldp_rib_group {
	struct fib_node		n;
	int			flags;
	list			entries;
} ldp_rib_group;

typedef struct _ldp_rib_entry {
	node			n;	/* Per rib grouplist */
	struct bird_mpls_entry	me; 	/* REMOTE MPLS labelstack */
	ip_addr			nexthop;	/* best route entry nexthop */
	struct iface		*iface;	/* best route Outgoing interface */
	uint16_t		flags;
	ldp_session		*ls;	/* Pointer to LDP session */
	ldp_rib_group		*rg;	/* Pointer to RIB group it belongs to */
	node			n_lss;	/* Per session RIB list */
	list			tlvs;	/* List of transitive TLVs */
} ldp_rib_entry;
#define	LDP_RE_DIRECT		0x02	/* entry is direct route from kernel */
#define	LDP_RE_NEXTHOP		0x04	/* entry is IGP route from gateway */
#define	LDP_RE_LDP		0x08	/* entry is candidate route from LDP neighbor */
#define	LDP_RE_HASLABEL		0x10	/* entry has LOCAL label allocated and announced */
#define LDP_RE_BEST		0x20	/* entry is currently selected as best route */
#define LDP_RE_INSTALLED	0x40	/* entry has (remote) MPLS label installed into RIB */

#define	LDP_RE_USED		(LDP_RE_DIRECT|LDP_RE_NEXTHOP|LDP_RE_LDP)

ldp_rib_group *ldp_rib_search_group(struct proto_ldp *pl, ip_addr *prefix, int plen);
void ldp_rt_withdraw_re(struct proto_ldp *pl, ldp_rib_entry *re);

void ldp_rib_set_best(struct proto_ldp *pl, ldp_rib_entry *re);
int ldp_rib_gc_entry(struct proto_ldp *pl, ldp_rib_entry *re);
void ldp_rib_link_session(struct proto_ldp *pl, ldp_rib_entry *re, ldp_session *ls);
void ldp_rib_unlink_session(struct proto_ldp *pl, ldp_rib_entry *re);
ldp_rib_entry *ldp_rib_findlink(struct proto_ldp *pl, ip_addr *prefix, int pxlen, ldp_session *ls);
ldp_rib_entry *ldp_rib_findlink_nh(struct proto_ldp *pl, ldp_rib_group *rg, ip_addr nexthop);

ldp_rib_entry *ldp_rib_add_entry(struct proto_ldp *pl, ip_addr *prefix, int pxlen, ldp_session *ls);
int ldp_rib_del_entry(struct proto_ldp *pl, ldp_rib_entry *re);
ldp_rib_group *ldp_rib_get_group(struct proto_ldp *pl, ip_addr *prefix, int pxlen);


typedef struct _ldp_addrmap {
	node			n_hash;	/* Hash with all nexthop addresses */
	node			n_lss;	/* Per session list */
	ip_addr			addr;	/* LSR interface address */
	struct _ldp_session	*ls;	/* Pointer to session to which nexthop belongs to */
} ldp_addrmap;

/* addrmap */
struct _ldp_session *ldp_addrmap_find_session(struct proto_ldp *pl, ip_addr addr);
int ldp_addrmap_add(struct proto_ldp *pl, ldp_session *ls, ip_addr addr);
int ldp_addrmap_del(struct proto_ldp *pl, ldp_session *ls, ip_addr addr);
/* LFIB */
int ldp_lib_init(struct proto_ldp *pl);
void ldp_lib_free(struct proto_ldp *pl, int last UNUSED);
u32 ldp_lib_flabel_add(struct proto_ldp *pl, struct proto *fp, service_relfunc relfunc, char *info, int ltype, int mpls_op);
int ldp_lib_flabel_del(struct proto_ldp *pl, u32 label);
/* RIB */
int ldp_rib_init(struct proto_ldp *pl);
void ldp_rib_free(struct proto_ldp *pl, int last UNUSED);


#endif

