/*
 *	BIRD -- MPLS Label Information Database
 *
 *	(c) 2013 Alexander V. Chernikov <melifaro@ipfw.ru>
 *
 *	Can be freely distributed and used under the terms of the 
 *	GNU GPL or Simplified BSD License.
 */

#ifndef _BIRD_KRT_MPLS_H_
#define _BIRD_KRT_MPLS_H_

#ifdef MPLS_SUPPORT

#define	MPLS_LABEL_MIN	16
#define	MPLS_LABEL_MAX	0xFFFFF

typedef struct _krt_lib_label {
	struct fib_node		n;
	struct bird_mpls_entry	me;
	uint16_t		type;
	uint16_t		flags;
	struct proto		*proto; /* pointer to protocol requesting label */
} krt_lib_label;

typedef struct _krt_lib_proto {
	struct fib_node		n;
	struct proto		*proto; /* pointer to protocol instance */
	service_relfunc		relfunc; /* pointer to label release function */
	int			labels;	/* number of labels requested */
} krt_lib_proto;

typedef struct _krt_lib_free_label {
	node			n;
	int			label;
} krt_lib_free_label;

typedef struct _krt_lib_database {
	struct fib		f;		/* fib to store label data */
	list			free_items;	/* List of items currently free */
	int			search_idx;
	int			current_label;	/* Current (used) label */
	struct fib		pr;		/* fib with protocol pointers */
	int			pr_count;	/* Number of protocols using LIB */
	struct service_hook	*sh;		/* Hook used to prived LIB service */
} krt_lib_database;

/* Operations supported by our implementation/kernel */
enum kmpls_op {
	KMPLS_OP_NONE,	/* Skip 0 */
	KMPLS_OP_POP,	/* Pop label */
	KMPLS_OP_PUSH,	/* Push label */
	KMPLS_OP_SWAP,	/* Swap label */
	KMPLS_OP_L3VPN,
	KMPLS_OP_NETGRAPH,	/* Forward packet to netgraph node */
};

char *krt_print_mplsop(enum kmpls_op op);
char *krt_print_mplslbl(u32 label);

/*
 * Request subclasses we use
 */
/* SRCLASS_LABEL */
#define SRSUB_LABEL_REQ		1
#define	SRSUB_LABEL_OPENDB	2
#define	SRSUB_LABEL_CLOSEDB	3

/*
 * Request label for given operation.
 */
struct krt_mpls_label_request
{
	struct service_request req;
	u16 op;		/* Label operation */
	u16 flags;	/* NOT USED */
	u32 data;	/* Additional data we need to specify */
};
struct krt_mpls_label_reply
{
	struct service_reply	repl;
	u32	label;
};
#define SRSUB_LABEL_REL	2
struct krt_mpls_label_release
{
	u32 label;
};

#endif

#endif

